/* fir-tools/src/fir2csv/200_load_filter.c
 *
 *  Copyright: ©2014, Laurence Withers
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



struct fir_filter_t*
load_filter(const char* fname)
{
    FILE* fp;
    int ret, pos, size;
    double coeff, * clist;
    char ch;
    struct fir_filter_t* fi;

    fp = fopen(fname, "r");
    if(!fp) {
        fprintf(stderr, APP_NAME ": unable to open \"%s\": %m.\n", fname);
        exit(1);
    }

    pos = 0;
    size = 1;
    clist = malloc(sizeof(double) * size);

    while(1) {
        ret = fscanf(fp, " %lf%c", &coeff, &ch);
        switch(ret) {
        default:
            if(ferror(fp)) {
                fprintf(stderr, APP_NAME ": error reading from \"%s\": %m.\n",
                    fname);
                exit(1);
            }
            goto done;

        case 2:
            if(!isspace(ch) && ch != ',') {
                fprintf(stderr, APP_NAME ": error reading coefficient %d "
                    "from file \"%s\".\n", pos + 1, fname);
                exit(1);
            }
            /* fall through */
        case 1:
            /* add to list */
            if(pos == size) {
                size <<= 1;
                clist = realloc(clist, sizeof(double) * size);
            }
            clist[pos++] = coeff;
            break;
        }
    }

  done:
    if(pos < 2) {
        fprintf(stderr, APP_NAME ": not enough coefficients in \"%s\".\n",
            fname);
        exit(1);
    }

    fi = fir_filter_new(pos, clist, fir_symmetry_none);
    if(!fi) {
        fprintf(stderr, APP_NAME ": error creating FIR filter: %m.\n");
        exit(1);
    }

    free(clist);
    return fi;
}



/* options for text editors
vim: expandtab:ts=4:sw=4
*/
