/* fir-tools/src/fir3db/999_main.c
 *
 *  Copyright: ©2014, Laurence Withers
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



void
usage(void)
{
    fputs("Usage:\n\n"
        "    " APP_NAME " [options] coeff.txt\n\n"
        "Options:\n"
        " -h, --help            Display this screen.\n"
        " -V, --version         Display version number.\n"
        " -g, --gain <gain>     Gain to search for (default: 0.5).\n"
    "", stdout);
}



const struct option options[] = {
    { "help", no_argument, 0, 'h' },
    { "version", no_argument, 0, 'V' },
    { "gain", no_argument, 0, 'g' },
    { 0, 0, 0, 0 }
};

const char* optstr = "g:hV";



int
main(int argc, char* argv[])
{
    struct fir_filter_t* fi;
    double gain = 0.5;
    char* endp;

    while(1) {
        switch(getopt_long(argc, argv, optstr, options, 0)) {
        case -1:
            goto opts_done;

        case '?':
            return 1;

        case 'h':
            usage();
            return 0;

        case 'V':
            fputs(APP_NAME " " VERSION "\n", stdout);
            return 0;

        case 'g':
            errno = 0;
            gain = strtod(optarg, &endp);
            if(errno || !endp || *endp || gain <= 0) {
                fputs(APP_NAME ": invalid gain. Expecting magnitude > 0.\n",
                    stderr);
                return 1;
            }
            break;
        }
    }
  opts_done:
    if(optind != (argc - 1)) {
        usage();
        return 1;
    }

    fi = load_filter(argv[optind]);
    fir3db(fi, gain);

    return 0;
}



/* options for text editors
vim: expandtab:ts=4:sw=4
*/
